/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *
 *******************************************************************************/
package org.fortiss.tooling.base.ui.editor.fx;

import static java.lang.Math.max;
import static org.fortiss.tooling.base.ui.editor.fx.ContextMenuUtil.createElementCompositionContext;
import static org.fortiss.tooling.base.ui.editor.fx.ContextMenuUtil.createPrototypeMenu;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramViewerFeatures;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.rectangular.RectangularResizableContentControllerBase;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeProvider;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IContentMVCBundle;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.kernel.extension.data.IElementCompositionContext;
import org.fortiss.tooling.kernel.service.IElementCompositorService;

import javafx.scene.Node;
import javafx.scene.control.MenuItem;

/**
 * {@link RectangularResizableContentControllerBase} with a listener mechanism for the layout of the
 * connected {@link EObject model element}.
 * 
 * @author hoelzl
 */
public abstract class EObjectBasedRectangularResizableContentControllerBase
		extends RectangularResizableContentControllerBase {
	/** {@link IModelChangeProvider} for this controller */
	private final LayoutModelElementModelChangeProvider layoutModelChangeProvider;

	/** Constructor. */
	public EObjectBasedRectangularResizableContentControllerBase(IContentMVCBundle mvcb) {
		super(mvcb);
		if(!(mvcb.getModel() instanceof ILayoutedModelElement)) {
			throw new IllegalArgumentException("Model element must be an instance of an EObject.");
		}
		ILayoutedModelElement lme = (ILayoutedModelElement)mvcb.getModel();
		this.layoutModelChangeProvider = new LayoutModelElementModelChangeProvider(lme);
	}

	/** Returns the {@link ILayoutedModelElement} model element. */
	protected ILayoutedModelElement getLayoutedME() {
		// wild cast works: see constructor check with exception
		return (ILayoutedModelElement)getModel();
	}

	/** {@inheritDoc} */
	@Override
	public IModelChangeProvider getModelChangeProvider() {
		return layoutModelChangeProvider;
	}

	/** {@inheritDoc} */
	@Override
	public List<MenuItem> contextMenuContributions(Node node, DiagramCoordinate diagramLocation) {
		// outer-most grid areas of the diagram are inaccessible
		DiagramViewerFeatures features = getViewer().getFeatures();
		double x = max(features.getHorizontalSpacing(), diagramLocation.getX());
		double y = max(features.getVerticalSpacing(), diagramLocation.getY());

		// wild cast works: see constructor exception
		EObject modelParent = (EObject)getModel();
		IElementCompositionContext edc = createElementCompositionContext(modelParent, x, y, false,
				getViewer().getFeatures().getCurrentZoomFactor());
		return createPrototypeMenu(modelParent, edc);
	}

	/** {@inheritDoc} */
	@Override
	public void delete() {
		IElementCompositorService.getInstance().decompose(getLayoutedME());
	}
}
